@@ -12,14 +12,14 @@ use crate::{
     messages::{Message, Variant},
     node::Node,
     routing::command::{self, Command},
-    section::{SectionAuthorityProvider, SectionKeyShare},
+    section::{ElderCandidates, SectionAuthorityProvider, SectionKeyShare},
     supermajority,
 };
 use bls_dkg::key_gen::{message::Message as DkgMessage, KeyGen};
 use hex_fmt::HexFmt;
 use itertools::Itertools;
 use serde::{Deserialize, Serialize};
-use sn_messaging::DstLocation;
+use sn_messaging::{DestInfo, DstLocation};
 use std::{
     collections::{BTreeSet, HashMap, VecDeque},
     fmt::{self, Debug, Formatter},
@@ -43,17 +43,17 @@ pub struct DkgKey {
 }
 
 impl DkgKey {
-    pub fn new(section_auth: &SectionAuthorityProvider, generation: u64) -> Self {
+    pub fn new(elder_candidates: &ElderCandidates, generation: u64) -> Self {
         // Calculate the hash without involving serialization to avoid having to return `Result`.
         let mut hasher = Sha3::v256();
         let mut hash = Digest256::default();
 
-        for peer in section_auth.peers() {
+        for peer in elder_candidates.peers() {
             hasher.update(&peer.name().0);
         }
 
-        hasher.update(&section_auth.prefix.name().0);
-        hasher.update(&section_auth.prefix.bit_count().to_le_bytes());
+        hasher.update(&elder_candidates.prefix.name().0);
+        hasher.update(&elder_candidates.prefix.bit_count().to_le_bytes());
         hasher.finalize(&mut hash);
 
         Self { hash, generation }
@@ -106,28 +106,31 @@ impl DkgVoter {
         &mut self,
         keypair: &Keypair,
         dkg_key: DkgKey,
-        section_auth: SectionAuthorityProvider,
+        elder_candidates: ElderCandidates,
     ) -> Vec<DkgCommand> {
         if self.sessions.contains_key(&dkg_key) {
-            trace!("DKG for {} already in progress", section_auth);
+            trace!("DKG for {:?} already in progress", elder_candidates);
             return vec![];
         }
 
         let name = crypto::name(&keypair.public);
-        let participant_index = if let Some(index) = section_auth.position(&name) {
+        let participant_index = if let Some(index) = elder_candidates.position(&name) {
             index
         } else {
             error!(
-                "DKG for {} failed to start: {} is not a participant",
-                section_auth, name
+                "DKG for {:?} failed to start: {} is not a participant",
+                elder_candidates, name
             );
             return vec![];
         };
 
         // Special case: only one participant.
-        if section_auth.elders.len() == 1 {
+        if elder_candidates.elders.len() == 1 {
             let secret_key_set = bls::SecretKeySet::random(0, &mut rand::thread_rng());
-
+            let section_auth = SectionAuthorityProvider::from_elder_candidates(
+                elder_candidates,
+                secret_key_set.public_keys(),
+            );
             return vec![DkgCommand::HandleOutcome {
                 section_auth,
                 outcome: SectionKeyShare {
@@ -138,16 +141,16 @@ impl DkgVoter {
             }];
         }
 
-        let threshold = supermajority(section_auth.elders.len()) - 1;
-        let participants = section_auth.elders.keys().copied().collect();
+        let threshold = supermajority(elder_candidates.elders.len()) - 1;
+        let participants = elder_candidates.elders.keys().copied().collect();
 
         match KeyGen::initialize(name, threshold, participants) {
             Ok((key_gen, message)) => {
-                trace!("DKG for {} starting", section_auth);
+                trace!("DKG for {:?} starting", elder_candidates);
 
                 let mut session = Session {
                     key_gen,
-                    section_auth,
+                    elder_candidates,
                     participant_index,
                     timer_token: 0,
                     failures: DkgFailureProofSet::default(),
@@ -175,7 +178,7 @@ impl DkgVoter {
             }
             Err(error) => {
                 // TODO: return a separate error here.
-                error!("DKG for {} failed to start: {}", section_auth, error);
+                error!("DKG for {:?} failed to start: {}", elder_candidates, error);
                 vec![]
             }
         }
@@ -223,7 +226,7 @@ impl DkgVoter {
 
 // Data for a DKG participant.
 struct Session {
-    section_auth: SectionAuthorityProvider,
+    elder_candidates: ElderCandidates,
     participant_index: usize,
     key_gen: KeyGen,
     timer_token: u64,
@@ -263,13 +266,13 @@ impl Session {
         commands
     }
 
-    fn recipients(&self) -> Vec<SocketAddr> {
-        self.section_auth
+    fn recipients(&self) -> Vec<(XorName, SocketAddr)> {
+        self.elder_candidates
             .elders
-            .values()
+            .iter()
             .enumerate()
             .filter(|(index, _)| *index != self.participant_index)
-            .map(|(_, addr)| *addr)
+            .map(|(_, (name, addr))| (*name, *addr))
             .collect()
     }
 
@@ -300,7 +303,7 @@ impl Session {
             return vec![];
         }
 
-        trace!("DKG for {} progressing", self.section_auth);
+        trace!("DKG for {:?} progressing", self.elder_candidates);
 
         match self.key_gen.timed_phase_transition(&mut rand::thread_rng()) {
             Ok(messages) => {
@@ -313,7 +316,7 @@ impl Session {
                 commands
             }
             Err(error) => {
-                trace!("DKG for {} failed: {}", self.section_auth, error);
+                trace!("DKG for {:?} failed: {}", self.elder_candidates, error);
                 self.report_failure(dkg_key, BTreeSet::new(), keypair)
             }
         }
@@ -336,15 +339,15 @@ impl Session {
         };
 
         // Less than 100% participation
-        if !participants.iter().eq(self.section_auth.elders.keys()) {
+        if !participants.iter().eq(self.elder_candidates.elders.keys()) {
             trace!(
-                "DKG for {} failed: unexpected participants: {:?}",
-                self.section_auth,
+                "DKG for {:?} failed: unexpected participants: {:?}",
+                self.elder_candidates,
                 participants.iter().format(", ")
             );
 
             let non_participants: BTreeSet<_> = self
-                .section_auth
+                .elder_candidates
                 .elders
                 .keys()
                 .filter_map(|elder| {
@@ -368,17 +371,24 @@ impl Session {
             .public_key_share(self.participant_index)
             != outcome.secret_key_share.public_key_share()
         {
-            trace!("DKG for {} failed: corrupted outcome", self.section_auth);
+            trace!(
+                "DKG for {:?} failed: corrupted outcome",
+                self.elder_candidates
+            );
             return self.report_failure(dkg_key, BTreeSet::new(), keypair);
         }
 
         trace!(
-            "DKG for {} complete: {:?}",
-            self.section_auth,
+            "DKG for {:?} complete: {:?}",
+            self.elder_candidates,
             outcome.public_key_set.public_key()
         );
 
         self.complete = true;
+        let section_auth = SectionAuthorityProvider::from_elder_candidates(
+            self.elder_candidates.clone(),
+            outcome.public_key_set.clone(),
+        );
 
         let outcome = SectionKeyShare {
             public_key_set: outcome.public_key_set,
@@ -387,7 +397,7 @@ impl Session {
         };
 
         vec![DkgCommand::HandleOutcome {
-            section_auth: self.section_auth.clone(),
+            section_auth,
             outcome,
         }]
     }
@@ -422,7 +432,7 @@ impl Session {
         proof: DkgFailureProof,
     ) -> Option<DkgCommand> {
         if !self
-            .section_auth
+            .elder_candidates
             .elders
             .contains_key(&crypto::name(&proof.public_key))
         {
@@ -441,7 +451,7 @@ impl Session {
     }
 
     fn check_failure_agreement(&mut self) -> Option<DkgCommand> {
-        if self.failures.has_agreement(&self.section_auth) {
+        if self.failures.has_agreement(&self.elder_candidates) {
             self.complete = true;
 
             Some(DkgCommand::HandleFailureAgreement(mem::take(
@@ -508,27 +518,27 @@ impl DkgFailureProofSet {
 
     // Check whether we have enough proofs to reach agreement on the failure. The contained proofs
     // are assumed valid.
-    fn has_agreement(&self, section_auth: &SectionAuthorityProvider) -> bool {
-        has_failure_agreement(section_auth.elders.len(), self.proofs.len())
+    fn has_agreement(&self, elder_candidates: &ElderCandidates) -> bool {
+        has_failure_agreement(elder_candidates.elders.len(), self.proofs.len())
     }
 
-    pub fn verify(&self, section_auth: &SectionAuthorityProvider, generation: u64) -> bool {
+    pub fn verify(&self, elder_candidates: &ElderCandidates, generation: u64) -> bool {
         let hash = failure_proof_hash(
-            &DkgKey::new(section_auth, generation),
+            &DkgKey::new(elder_candidates, generation),
             &self.non_participants,
         );
         let votes = self
             .proofs
             .iter()
             .filter(|proof| {
-                section_auth
+                elder_candidates
                     .elders
                     .contains_key(&crypto::name(&proof.public_key))
+                    && proof.public_key.verify(&hash, &proof.signature).is_ok()
             })
-            .filter(|proof| proof.public_key.verify(&hash, &proof.signature).is_ok())
             .count();
 
-        has_failure_agreement(section_auth.elders.len(), votes)
+        has_failure_agreement(elder_candidates.elders.len(), votes)
     }
 }
 
@@ -595,7 +605,7 @@ impl Backlog {
 #[derive(Debug)]
 pub(crate) enum DkgCommand {
     SendMessage {
-        recipients: Vec<SocketAddr>,
+        recipients: Vec<(XorName, SocketAddr)>,
         dkg_key: DkgKey,
         message: DkgMessage,
     },
@@ -608,7 +618,7 @@ pub(crate) enum DkgCommand {
         outcome: SectionKeyShare,
     },
     SendFailureObservation {
-        recipients: Vec<SocketAddr>,
+        recipients: Vec<(XorName, SocketAddr)>,
         dkg_key: DkgKey,
         proof: DkgFailureProof,
         non_participants: BTreeSet<XorName>,
@@ -617,7 +627,7 @@ pub(crate) enum DkgCommand {
 }
 
 impl DkgCommand {
-    fn into_command(self, node: &Node) -> Result<Command> {
+    fn into_command(self, node: &Node, key: bls::PublicKey) -> Result<Command> {
         match self {
             Self::SendMessage {
                 recipients,
@@ -625,12 +635,16 @@ impl DkgCommand {
                 message,
             } => {
                 let variant = Variant::DkgMessage { dkg_key, message };
-                let message = Message::single_src(node, DstLocation::Direct, variant, None, None)?;
+                let message = Message::single_src(node, DstLocation::Direct, variant, None)?;
 
                 Ok(Command::send_message_to_nodes(
-                    &recipients,
+                    recipients.clone(),
                     recipients.len(),
                     message.to_bytes(),
+                    DestInfo {
+                        dest: XorName::random(),
+                        dest_section_pk: key,
+                    },
                 ))
             }
             Self::ScheduleTimeout { duration, token } => {
@@ -654,12 +668,16 @@ impl DkgCommand {
                     proof,
                     non_participants,
                 };
-                let message = Message::single_src(node, DstLocation::Direct, variant, None, None)?;
+                let message = Message::single_src(node, DstLocation::Direct, variant, None)?;
 
                 Ok(Command::send_message_to_nodes(
-                    &recipients,
+                    recipients.clone(),
                     recipients.len(),
                     message.to_bytes(),
+                    DestInfo {
+                        dest: XorName::random(),
+                        dest_section_pk: key,
+                    },
                 ))
             }
             Self::HandleFailureAgreement(proofs) => Ok(Command::HandleDkgFailure(proofs)),
@@ -668,21 +686,21 @@ impl DkgCommand {
 }
 
 pub(crate) trait DkgCommands {
-    fn into_commands(self, node: &Node) -> Result<Vec<Command>>;
+    fn into_commands(self, node: &Node, key: bls::PublicKey) -> Result<Vec<Command>>;
 }
 
 impl DkgCommands for Vec<DkgCommand> {
-    fn into_commands(self, node: &Node) -> Result<Vec<Command>> {
+    fn into_commands(self, node: &Node, key: bls::PublicKey) -> Result<Vec<Command>> {
         self.into_iter()
-            .map(|command| command.into_command(node))
+            .map(|command| command.into_command(node, key))
             .collect()
     }
 }
 
 impl DkgCommands for Option<DkgCommand> {
-    fn into_commands(self, node: &Node) -> Result<Vec<Command>> {
+    fn into_commands(self, node: &Node, key: bls::PublicKey) -> Result<Vec<Command>> {
         self.into_iter()
-            .map(|command| command.into_command(node))
+            .map(|command| command.into_command(node, key))
             .collect()
     }
 }
@@ -710,11 +728,10 @@ mod tests {
             crypto::gen_keypair(&Prefix::default().range_inclusive(), MIN_ADULT_AGE),
             gen_addr(),
         );
-        let section_auth =
-            SectionAuthorityProvider::new(iter::once(node.peer()), Prefix::default());
-        let dkg_key = DkgKey::new(&section_auth, 0);
+        let elder_candidates = ElderCandidates::new(iter::once(node.peer()), Prefix::default());
+        let dkg_key = DkgKey::new(&elder_candidates, 0);
 
-        let commands = voter.start(&node.keypair, dkg_key, section_auth);
+        let commands = voter.start(&node.keypair, dkg_key, elder_candidates);
         assert_matches!(&commands[..], &[DkgCommand::HandleOutcome { .. }]);
     }
 
@@ -733,9 +750,9 @@ mod tests {
         let mut rng = SmallRng::seed_from_u64(seed);
         let mut messages = Vec::new();
 
-        let section_auth =
-            SectionAuthorityProvider::new(nodes.iter().map(Node::peer), Prefix::default());
-        let dkg_key = DkgKey::new(&section_auth, 0);
+        let elder_candidates =
+            ElderCandidates::new(nodes.iter().map(Node::peer), Prefix::default());
+        let dkg_key = DkgKey::new(&elder_candidates, 0);
 
         let mut actors: HashMap<_, _> = nodes
             .into_iter()
@@ -743,9 +760,10 @@ mod tests {
             .collect();
 
         for actor in actors.values_mut() {
-            let commands = actor
-                .voter
-                .start(&actor.node.keypair, dkg_key, section_auth.clone());
+            let commands =
+                actor
+                    .voter
+                    .start(&actor.node.keypair, dkg_key, elder_candidates.clone());
 
             for command in commands {
                 messages.extend(actor.handle(command, &dkg_key))
@@ -810,7 +828,7 @@ mod tests {
                     assert_eq!(dkg_key, *expected_dkg_key);
                     recipients
                         .into_iter()
-                        .map(|addr| (addr, message.clone()))
+                        .map(|addr| (addr.1, message.clone()))
                         .collect()
                 }
                 DkgCommand::HandleOutcome { outcome, .. } => {
