@@ -1,4 +1,4 @@
-// Copyright 2020 MaidSafe.net limited.
+// Copyright 2021 MaidSafe.net limited.
 //
 // This SAFE Network Software is licensed to you under The General Public License (GPL), version 3.
 // Unless required by applicable law or agreed to in writing, the SAFE Network Software distributed
@@ -6,12 +6,11 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-use super::{
-    core::{RESOURCE_PROOF_DATA_SIZE, RESOURCE_PROOF_DIFFICULTY},
-    Comm, Command, Core, Dispatcher,
-};
+use super::{Comm, Command, Core, Dispatcher};
+use crate::agreement::test_utils::{prove, proven};
+use crate::agreement::{Proposal, Proven};
+use crate::routing::core::{RESOURCE_PROOF_DATA_SIZE, RESOURCE_PROOF_DIFFICULTY};
 use crate::{
-    agreement::{test_utils::*, Proposal, Proven},
     crypto,
     event::Event,
     messages::{JoinRequest, Message, PlainMessage, ResourceProofResponse, Variant, VerifyStatus},
@@ -30,11 +29,12 @@ use assert_matches::assert_matches;
 use bls_signature_aggregator::Proof;
 use bytes::Bytes;
 use resource_proof::ResourceProof;
+use sn_data_types::{Keypair, PublicKey};
 use sn_messaging::{
     location::{Aggregation, Itinerary},
-    node::NodeMessage,
+    node::RoutingMsg,
     section_info::{GetSectionResponse, Message as SectionInfoMsg},
-    DstLocation, MessageType, SrcLocation,
+    DestInfo, DstLocation, MessageType, SrcLocation,
 };
 use std::{
     collections::{BTreeSet, HashSet},
@@ -56,13 +56,18 @@ async fn receive_matching_get_section_request_as_elder() -> Result<()> {
         crypto::gen_keypair(&Prefix::default().range_inclusive(), MIN_ADULT_AGE),
         gen_addr(),
     );
+    let new_node_name = new_node.name();
 
-    let message = SectionInfoMsg::GetSectionQuery(new_node.name());
+    let message = SectionInfoMsg::GetSectionQuery(PublicKey::from(new_node.keypair.public));
 
     let mut commands = dispatcher
         .handle_command(Command::HandleSectionInfoMsg {
             sender: new_node.addr,
             message,
+            dest_info: DestInfo {
+                dest: new_node_name,
+                dest_section_pk: bls::SecretKey::random().public_key(),
+            },
         })
         .await?
         .into_iter();
@@ -71,11 +76,11 @@ async fn receive_matching_get_section_request_as_elder() -> Result<()> {
         commands.next(),
         Some(Command::SendMessage {
             recipients,
-            message: MessageType::SectionInfo(message), ..
-        }) => (recipients, message)
+            message: MessageType::SectionInfo{ msg, .. }, ..
+        }) => (recipients, msg)
     );
 
-    assert_eq!(recipients, [new_node.addr]);
+    assert_eq!(recipients, [(new_node.name(), new_node.addr)]);
 
     assert_matches!(
         message,
@@ -88,26 +93,43 @@ async fn receive_matching_get_section_request_as_elder() -> Result<()> {
 #[tokio::test]
 async fn receive_mismatching_get_section_request_as_adult() -> Result<()> {
     let good_prefix = Prefix::default().pushed(false);
-    let bad_prefix = Prefix::default().pushed(true);
 
     let sk_set = SecretKeySet::random();
     let (section_auth, _) = gen_section_authority_provider(good_prefix, ELDER_SIZE);
-    let elders_addrs: Vec<_> = section_auth.elders.values().copied().collect();
+    let elders_addrs: Vec<_> = section_auth
+        .elders()
+        .iter()
+        .map(|(name, addr)| (*name, *addr))
+        .collect();
     let (section, _) = create_section(&sk_set, &section_auth)?;
 
     let node = create_node(MIN_ADULT_AGE);
     let state = Core::new(node, section, None, mpsc::unbounded_channel().0);
     let dispatcher = Dispatcher::new(state, create_comm().await?);
 
-    let new_node_name = bad_prefix.substituted_in(rand::random());
+    let mut rng = rand::thread_rng();
+    let mut keypair = Keypair::new_ed25519(&mut rng);
+    let mut random_pk = keypair.public_key();
+    let mut new_node_name = XorName::from(random_pk);
+
+    while new_node_name.bit(0) {
+        keypair = Keypair::new_ed25519(&mut rng);
+        random_pk = keypair.public_key();
+        new_node_name = XorName::from(random_pk);
+    }
+
     let new_node_addr = gen_addr();
 
-    let message = SectionInfoMsg::GetSectionQuery(new_node_name);
+    let message = SectionInfoMsg::GetSectionQuery(random_pk);
 
     let mut commands = dispatcher
         .handle_command(Command::HandleSectionInfoMsg {
             sender: new_node_addr,
             message,
+            dest_info: DestInfo {
+                dest: new_node_name,
+                dest_section_pk: bls::SecretKey::random().public_key(),
+            },
         })
         .await?
         .into_iter();
@@ -116,11 +138,11 @@ async fn receive_mismatching_get_section_request_as_adult() -> Result<()> {
         commands.next(),
         Some(Command::SendMessage {
             recipients,
-            message: MessageType::SectionInfo(message), ..
-        }) => (recipients, message)
+            message: MessageType::SectionInfo { msg, .. }, ..
+        }) => (recipients, msg)
     );
 
-    assert_eq!(recipients, [new_node_addr]);
+    assert_eq!(recipients, [(new_node_name, new_node_addr)]);
     assert_matches!(
         message,
         SectionInfoMsg::GetSectionResponse(GetSectionResponse::Redirect(addrs)) => {
@@ -138,6 +160,7 @@ async fn receive_mismatching_get_section_request_as_adult() -> Result<()> {
 #[tokio::test]
 async fn receive_join_request_without_resource_proof_response() -> Result<()> {
     let node = create_node(FIRST_SECTION_MIN_AGE);
+    let node_name = node.name();
     let state = Core::first_node(node, mpsc::unbounded_channel().0)?;
     let dispatcher = Dispatcher::new(state, create_comm().await?);
 
@@ -156,19 +179,22 @@ async fn receive_join_request_without_resource_proof_response() -> Result<()> {
             resource_proof_response: None,
         })),
         None,
-        None,
     )?;
     let mut commands = dispatcher
         .handle_command(Command::HandleMessage {
             sender: Some(new_node.addr),
             message,
+            dest_info: DestInfo {
+                dest: node_name,
+                dest_section_pk: section_key,
+            },
         })
         .await?
         .into_iter();
 
     let response_message = assert_matches!(
         commands.next(),
-        Some(Command::SendMessage { message: MessageType::NodeMessage(NodeMessage(message)), .. }) => message
+        Some(Command::SendMessage { message: MessageType::Routing { msg: RoutingMsg(message), .. }, .. }) => message
     );
     let response_message = Message::from_bytes(Bytes::from(response_message))?;
 
@@ -183,6 +209,7 @@ async fn receive_join_request_without_resource_proof_response() -> Result<()> {
 #[tokio::test]
 async fn receive_join_request_with_resource_proof_response() -> Result<()> {
     let node = create_node(FIRST_SECTION_MIN_AGE);
+    let node_name = node.name();
     let state = Core::first_node(node, mpsc::unbounded_channel().0)?;
     let dispatcher = Dispatcher::new(state, create_comm().await?);
 
@@ -215,13 +242,16 @@ async fn receive_join_request_with_resource_proof_response() -> Result<()> {
             }),
         })),
         None,
-        None,
     )?;
 
     let commands = dispatcher
         .handle_command(Command::HandleMessage {
             sender: Some(new_node.addr),
             message,
+            dest_info: DestInfo {
+                dest: node_name,
+                dest_section_pk: section_key,
+            },
         })
         .await?
         .into_iter();
@@ -259,6 +289,7 @@ async fn receive_join_request_from_relocated_node() -> Result<()> {
 
     let (section, section_key_share) = create_section(&sk_set, &section_auth)?;
     let node = nodes.remove(0);
+    let node_name = node.name();
     let state = Core::new(
         node,
         section,
@@ -316,13 +347,16 @@ async fn receive_join_request_from_relocated_node() -> Result<()> {
             resource_proof_response: None,
         })),
         None,
-        None,
     )?;
 
     let commands = dispatcher
         .handle_command(Command::HandleMessage {
             sender: Some(relocated_node.addr),
             message: join_request,
+            dest_info: DestInfo {
+                dest: node_name,
+                dest_section_pk: section_key,
+            },
         })
         .await?;
 
@@ -356,7 +390,7 @@ async fn aggregate_proposals() -> Result<()> {
     let (section, section_key_share) = create_section(&sk_set, &section_auth)?;
     let state = Core::new(
         nodes[0].clone(),
-        section,
+        section.clone(),
         Some(section_key_share),
         mpsc::unbounded_channel().0,
     );
@@ -380,13 +414,16 @@ async fn aggregate_proposals() -> Result<()> {
                 proof_share,
             },
             None,
-            None,
         )?;
 
         let commands = dispatcher
             .handle_command(Command::HandleMessage {
                 message,
-                sender: Some(nodes[0].addr),
+                sender: Some(nodes[index].addr),
+                dest_info: DestInfo {
+                    dest: nodes[0].name(),
+                    dest_section_pk: *section.chain().last_key(),
+                },
             })
             .await?;
         assert!(commands.is_empty());
@@ -405,12 +442,15 @@ async fn aggregate_proposals() -> Result<()> {
             proof_share,
         },
         None,
-        None,
     )?;
     let mut commands = dispatcher
         .handle_command(Command::HandleMessage {
             message,
             sender: Some(nodes[THRESHOLD].addr),
+            dest_info: DestInfo {
+                dest: nodes[0].name(),
+                dest_section_pk: *section.chain().last_key(),
+            },
         })
         .await?
         .into_iter();
@@ -459,8 +499,11 @@ async fn handle_agreement_on_online_of_elder_candidate() -> Result<()> {
     // Creates nodes where everybody has age 6 except one has 5.
     let mut nodes: Vec<_> = gen_sorted_nodes(&Prefix::default(), ELDER_SIZE, true);
 
-    let section_auth =
-        SectionAuthorityProvider::new(nodes.iter().map(Node::peer), Prefix::default());
+    let section_auth = SectionAuthorityProvider::new(
+        nodes.iter().map(Node::peer),
+        Prefix::default(),
+        sk_set.public_keys(),
+    );
     let proven_section_auth = proven(sk_set.secret_key(), section_auth.clone())?;
 
     let mut section = Section::new(*chain.root_key(), chain, proven_section_auth)?;
@@ -514,23 +557,28 @@ async fn handle_agreement_on_online_of_elder_candidate() -> Result<()> {
         let (recipients, message) = match command {
             Command::SendMessage {
                 recipients,
-                message: MessageType::NodeMessage(NodeMessage(msg_bytes)),
+                message:
+                    MessageType::Routing {
+                        msg: RoutingMsg(msg_bytes),
+                        ..
+                    },
                 ..
             } => (recipients, Message::from_bytes(Bytes::from(msg_bytes))?),
             _ => continue,
         };
 
-        let actual_section_auth = match message.variant() {
-            Variant::DkgStart { section_auth, .. } => section_auth,
+        let actual_elder_candidates = match message.variant() {
+            Variant::DkgStart {
+                elder_candidates, ..
+            } => elder_candidates,
             _ => continue,
         };
-        itertools::assert_equal(actual_section_auth.peers(), expected_new_elders.clone());
+        itertools::assert_equal(actual_elder_candidates.peers(), expected_new_elders.clone());
 
         let expected_dkg_start_recipients: Vec<_> = expected_new_elders
             .iter()
             .filter(|peer| *peer.name() != node_name)
-            .map(Peer::addr)
-            .copied()
+            .map(|peer| (*peer.name(), *peer.addr()))
             .collect();
         assert_eq!(recipients, expected_dkg_start_recipients);
 
@@ -570,7 +618,11 @@ async fn handle_online_command(
         let (message, recipients) = match command {
             Command::SendMessage {
                 recipients,
-                message: MessageType::NodeMessage(NodeMessage(msg_bytes)),
+                message:
+                    MessageType::Routing {
+                        msg: RoutingMsg(msg_bytes),
+                        ..
+                    },
                 ..
             } => (Message::from_bytes(Bytes::from(msg_bytes))?, recipients),
             _ => continue,
@@ -582,7 +634,7 @@ async fn handle_online_command(
                 ..
             } => {
                 assert_eq!(proven_section_auth.value, *section_auth);
-                assert_eq!(recipients, [*peer.addr()]);
+                assert_eq!(recipients, [(*peer.name(), *peer.addr())]);
                 status.node_approval_sent = true;
             }
             Variant::Relocate(details) => {
@@ -590,7 +642,7 @@ async fn handle_online_command(
                     continue;
                 }
 
-                assert_eq!(recipients, [*peer.addr()]);
+                assert_eq!(recipients, [(*peer.name(), *peer.addr())]);
 
                 status.relocate_details = Some(details.clone());
             }
@@ -756,14 +808,20 @@ async fn handle_agreement_on_offline_of_elder() -> Result<()> {
         let (recipients, message) = match command {
             Command::SendMessage {
                 recipients,
-                message: MessageType::NodeMessage(NodeMessage(msg_bytes)),
+                message:
+                    MessageType::Routing {
+                        msg: RoutingMsg(msg_bytes),
+                        ..
+                    },
                 ..
             } => (recipients, Message::from_bytes(Bytes::from(msg_bytes))?),
             _ => continue,
         };
 
-        let actual_section_auth = match message.variant() {
-            Variant::DkgStart { section_auth, .. } => section_auth,
+        let actual_elder_candidates = match message.variant() {
+            Variant::DkgStart {
+                elder_candidates, ..
+            } => elder_candidates,
             _ => continue,
         };
 
@@ -772,13 +830,12 @@ async fn handle_agreement_on_offline_of_elder() -> Result<()> {
             .filter(|peer| *peer != remove_peer)
             .chain(iter::once(existing_peer))
             .collect();
-        itertools::assert_equal(actual_section_auth.peers(), expected_new_elders.clone());
+        itertools::assert_equal(actual_elder_candidates.peers(), expected_new_elders.clone());
 
         let expected_dkg_start_recipients: Vec<_> = expected_new_elders
             .iter()
             .filter(|peer| *peer.name() != node_name)
-            .map(Peer::addr)
-            .copied()
+            .map(|peer| (*peer.name(), *peer.addr()))
             .collect();
         assert_eq!(recipients, expected_dkg_start_recipients);
 
@@ -798,8 +855,7 @@ async fn handle_agreement_on_offline_of_elder() -> Result<()> {
         .await
         .section()
         .authority_provider()
-        .elders
-        .contains_key(remove_peer.name()));
+        .contains_elder(remove_peer.name()));
 
     Ok(())
 }
@@ -831,16 +887,15 @@ async fn handle_untrusted_message(source: UntrustedMessageSource) -> Result<()>
             // When the untrusted message is sent from a single peer, we should bounce it back to
             // that peer.
             let sender = *section_auth
-                .elders
-                .values()
-                .next()
+                .addresses()
+                .get(0)
                 .expect("section_auth is empty");
             (Some(sender), vec![sender])
         }
         UntrustedMessageSource::Accumulation => {
             // When the untrusted message is the result of message accumulation, we should bounce
             // it to our elders.
-            (None, section_auth.elders.values().copied().collect())
+            (None, section_auth.addresses())
         }
     };
 
@@ -849,7 +904,7 @@ async fn handle_untrusted_message(source: UntrustedMessageSource) -> Result<()>
 
     let node = create_node(MIN_ADULT_AGE);
     let node_name = node.name();
-    let state = Core::new(node, section, None, mpsc::unbounded_channel().0);
+    let state = Core::new(node, section.clone(), None, mpsc::unbounded_channel().0);
     let dispatcher = Dispatcher::new(state, create_comm().await?);
 
     let sk1 = bls::SecretKey::random();
@@ -876,6 +931,10 @@ async fn handle_untrusted_message(source: UntrustedMessageSource) -> Result<()>
         .handle_command(Command::HandleMessage {
             message: original_message.clone(),
             sender,
+            dest_info: DestInfo {
+                dest: node_name,
+                dest_section_pk: *section.chain().last_key(),
+            },
         })
         .await?;
 
@@ -884,7 +943,11 @@ async fn handle_untrusted_message(source: UntrustedMessageSource) -> Result<()>
     for command in commands {
         let (recipients, message) = if let Command::SendMessage {
             recipients,
-            message: MessageType::NodeMessage(NodeMessage(msg_bytes)),
+            message:
+                MessageType::Routing {
+                    msg: RoutingMsg(msg_bytes),
+                    ..
+                },
             ..
         } = command
         {
@@ -893,10 +956,16 @@ async fn handle_untrusted_message(source: UntrustedMessageSource) -> Result<()>
             continue;
         };
 
-        if let Variant::BouncedUntrustedMessage(bounced_message) = message.variant() {
-            assert_eq!(recipients, expected_recipients);
-            assert_eq!(**bounced_message, original_message);
-            assert_eq!(message.dst_key(), Some(&pk0));
+        if let Variant::BouncedUntrustedMessage { msg, dest_info } = message.variant() {
+            assert_eq!(
+                recipients
+                    .into_iter()
+                    .map(|recp| recp.1)
+                    .collect::<Vec<_>>(),
+                expected_recipients
+            );
+            assert_eq!(**msg, original_message);
+            assert_eq!(dest_info.dest_section_pk, pk0);
 
             bounce_sent = true;
         }
@@ -927,6 +996,7 @@ async fn handle_bounced_untrusted_message() -> Result<()> {
     let section_key_share = create_section_key_share(&sk1_set, 0);
 
     let node = nodes.remove(0);
+    let node_name = node.name();
 
     // Create the original message whose bounce we want to test. Attach a proof that starts
     // at `pk1`.
@@ -964,19 +1034,26 @@ async fn handle_bounced_untrusted_message() -> Result<()> {
     );
     let dispatcher = Dispatcher::new(state, create_comm().await?);
 
+    let dest_info = DestInfo {
+        dest: node_name,
+        dest_section_pk: pk0,
+    };
     // Create the bounced message, indicating the last key the peer knows is `pk0`
     let bounced_message = Message::single_src(
         &other_node,
         DstLocation::Direct,
-        Variant::BouncedUntrustedMessage(Box::new(original_message)),
+        Variant::BouncedUntrustedMessage {
+            msg: Box::new(original_message),
+            dest_info: dest_info.clone(),
+        },
         None,
-        Some(pk0),
     )?;
 
     let commands = dispatcher
         .handle_command(Command::HandleMessage {
             message: bounced_message,
             sender: Some(other_node.addr),
+            dest_info,
         })
         .await?;
 
@@ -986,7 +1063,11 @@ async fn handle_bounced_untrusted_message() -> Result<()> {
         let (recipients, message) = match command {
             Command::SendMessage {
                 recipients,
-                message: MessageType::NodeMessage(NodeMessage(msg_bytes)),
+                message:
+                    MessageType::Routing {
+                        msg: RoutingMsg(msg_bytes),
+                        ..
+                    },
                 ..
             } => (recipients, Message::from_bytes(Bytes::from(msg_bytes))?),
             _ => continue,
@@ -994,7 +1075,7 @@ async fn handle_bounced_untrusted_message() -> Result<()> {
 
         match message.variant() {
             Variant::UserMessage(content) => {
-                assert_eq!(recipients, [other_node.addr]);
+                assert_eq!(recipients, [(other_node.name(), other_node.addr)]);
                 assert_eq!(*content, original_message_content);
                 assert_eq!(*message.proof_chain()?, chain);
 
@@ -1029,11 +1110,13 @@ async fn handle_sync() -> Result<()> {
     let (event_tx, mut event_rx) = mpsc::unbounded_channel();
     let section_key_share = create_section_key_share(&sk1_set, 0);
     let node = nodes.remove(0);
+    let node_name = node.name();
     let state = Core::new(node, old_section, Some(section_key_share), event_tx);
     let dispatcher = Dispatcher::new(state, create_comm().await?);
 
     // Create new `Section` as a successor to the previous one.
-    let sk2 = bls::SecretKey::random();
+    let sk2_set = SecretKeySet::random();
+    let sk2 = sk2_set.secret_key();
     let pk2 = sk2.public_key();
     let pk2_signature = sk1_set.secret_key().sign(bincode::serialize(&pk2)?);
     chain.insert(&pk1, pk2, pk2_signature)?;
@@ -1045,12 +1128,13 @@ async fn handle_sync() -> Result<()> {
     let new_section_auth = SectionAuthorityProvider::new(
         old_section_auth
             .peers()
-            .take(old_section_auth.elders.len() - 1)
+            .take(old_section_auth.elder_count() - 1)
             .chain(iter::once(new_peer)),
         old_section_auth.prefix,
+        sk2_set.public_keys(),
     );
-    let new_section_elders: BTreeSet<_> = new_section_auth.elders.keys().copied().collect();
-    let proven_new_section_auth = proven(&sk2, new_section_auth)?;
+    let new_section_elders: BTreeSet<_> = new_section_auth.names();
+    let proven_new_section_auth = proven(sk2, new_section_auth)?;
     let new_section = Section::new(pk0, chain, proven_new_section_auth)?;
 
     // Create the `Sync` message containing the new `Section`.
@@ -1058,11 +1142,10 @@ async fn handle_sync() -> Result<()> {
         &old_node,
         DstLocation::Direct,
         Variant::Sync {
-            section: new_section,
+            section: new_section.clone(),
             network: Network::new(),
         },
         None,
-        None,
     )?;
 
     // Handle the message.
@@ -1070,6 +1153,10 @@ async fn handle_sync() -> Result<()> {
         .handle_command(Command::HandleMessage {
             message,
             sender: Some(old_node.addr),
+            dest_info: DestInfo {
+                dest: node_name,
+                dest_section_pk: *new_section.chain().last_key(),
+            },
         })
         .await?;
 
@@ -1114,6 +1201,7 @@ async fn handle_untrusted_sync() -> Result<()> {
 
     let (event_tx, mut event_rx) = mpsc::unbounded_channel();
     let node = create_node(MIN_ADULT_AGE);
+    let node_name = node.name();
     let state = Core::new(node, old_section, None, event_tx);
     let dispatcher = Dispatcher::new(state, create_comm().await?);
 
@@ -1122,17 +1210,20 @@ async fn handle_untrusted_sync() -> Result<()> {
         &sender,
         DstLocation::Direct,
         Variant::Sync {
-            section: new_section,
+            section: new_section.clone(),
             network: Network::new(),
         },
         None,
-        None,
     )?;
 
     let commands = dispatcher
         .handle_command(Command::HandleMessage {
             message: orig_message.clone(),
             sender: Some(sender.addr),
+            dest_info: DestInfo {
+                dest: node_name,
+                dest_section_pk: *new_section.chain().last_key(),
+            },
         })
         .await?;
 
@@ -1142,16 +1233,20 @@ async fn handle_untrusted_sync() -> Result<()> {
         let (recipients, message) = match command {
             Command::SendMessage {
                 recipients,
-                message: MessageType::NodeMessage(NodeMessage(msg_bytes)),
+                message:
+                    MessageType::Routing {
+                        msg: RoutingMsg(msg_bytes),
+                        ..
+                    },
                 ..
             } => (recipients, Message::from_bytes(Bytes::from(msg_bytes))?),
             _ => continue,
         };
 
         match message.variant() {
-            Variant::BouncedUntrustedMessage(bounced_message) => {
-                assert_eq!(**bounced_message, orig_message);
-                assert_eq!(recipients, [sender.addr]);
+            Variant::BouncedUntrustedMessage { msg, .. } => {
+                assert_eq!(**msg, orig_message);
+                assert_eq!(recipients, [(sender.name(), sender.addr)]);
                 bounce_sent = true;
             }
             _ => continue,
@@ -1190,6 +1285,7 @@ async fn handle_bounced_untrusted_sync() -> Result<()> {
 
     let (event_tx, _) = mpsc::unbounded_channel();
     let node = nodes.remove(0);
+    let node_name = node.name();
     let section_key_share = create_section_key_share(&sk2_set, 0);
     let state = Core::new(
         node.clone(),
@@ -1207,22 +1303,29 @@ async fn handle_bounced_untrusted_sync() -> Result<()> {
             network: Network::new(),
         },
         None,
-        None,
     )?;
 
+    let dest_info = DestInfo {
+        dest: node_name,
+        dest_section_pk: pk0,
+    };
+
     let sender = create_node(MIN_ADULT_AGE);
     let bounced_message = Message::single_src(
         &sender,
         DstLocation::Node(node.name()),
-        Variant::BouncedUntrustedMessage(Box::new(orig_message)),
+        Variant::BouncedUntrustedMessage {
+            msg: Box::new(orig_message),
+            dest_info: dest_info.clone(),
+        },
         None,
-        Some(pk0),
     )?;
 
     let commands = dispatcher
         .handle_command(Command::HandleMessage {
             message: bounced_message,
             sender: Some(sender.addr),
+            dest_info,
         })
         .await?;
 
@@ -1232,7 +1335,11 @@ async fn handle_bounced_untrusted_sync() -> Result<()> {
         let (recipients, message) = match command {
             Command::SendMessage {
                 recipients,
-                message: MessageType::NodeMessage(NodeMessage(msg_bytes)),
+                message:
+                    MessageType::Routing {
+                        msg: RoutingMsg(msg_bytes),
+                        ..
+                    },
                 ..
             } => (recipients, Message::from_bytes(Bytes::from(msg_bytes))?),
             _ => continue,
@@ -1240,7 +1347,7 @@ async fn handle_bounced_untrusted_sync() -> Result<()> {
 
         match message.variant() {
             Variant::Sync { section, .. } => {
-                assert_eq!(recipients, [sender.addr]);
+                assert_eq!(recipients, [(sender.name(), sender.addr)]);
                 assert!(section.chain().has_key(&pk0));
                 message_resent = true;
             }
@@ -1303,13 +1410,22 @@ async fn relocation(relocated_peer_role: RelocatedPeerRole) -> Result<()> {
         let (recipients, message) = match command {
             Command::SendMessage {
                 recipients,
-                message: MessageType::NodeMessage(NodeMessage(msg_bytes)),
+                message:
+                    MessageType::Routing {
+                        msg: RoutingMsg(msg_bytes),
+                        ..
+                    },
                 ..
             } => (recipients, Message::from_bytes(Bytes::from(msg_bytes))?),
             _ => continue,
         };
 
-        if recipients != [*relocated_peer.addr()] {
+        if recipients
+            .into_iter()
+            .map(|recp| recp.1)
+            .collect::<Vec<_>>()
+            != [*relocated_peer.addr()]
+        {
             continue;
         }
         match relocated_peer_role {
@@ -1360,11 +1476,12 @@ async fn message_to_self(dst: MessageDst) -> Result<()> {
     let peer = node.peer();
     let state = Core::first_node(node, mpsc::unbounded_channel().0)?;
     let dispatcher = Dispatcher::new(state, create_comm().await?);
+    let section_name = XorName::random();
 
     let src = SrcLocation::Node(*peer.name());
-    let dst = match dst {
-        MessageDst::Node => DstLocation::Node(*peer.name()),
-        MessageDst::Section => DstLocation::Section(rand::random()),
+    let (dst, dst_name) = match dst {
+        MessageDst::Node => (DstLocation::Node(*peer.name()), *peer.name()),
+        MessageDst::Section => (DstLocation::Section(section_name), section_name),
     };
     let content = Bytes::from_static(b"hello");
 
@@ -1380,10 +1497,11 @@ async fn message_to_self(dst: MessageDst) -> Result<()> {
         })
         .await?;
 
-    assert_matches!(&commands[..], [Command::HandleMessage { sender, message }] => {
+    assert_matches!(&commands[..], [Command::HandleMessage { sender, message, dest_info }] => {
         assert_eq!(sender.as_ref(), Some(peer.addr()));
         assert_eq!(message.src().src_location(), src);
         assert_eq!(message.dst(), &dst);
+        assert_eq!(dest_info.dest, dst_name);
         assert_matches!(
             message.variant(),
             Variant::UserMessage(actual_content) if actual_content == &content
@@ -1410,6 +1528,7 @@ async fn handle_elders_update() -> Result<()> {
     let section_auth0 = SectionAuthorityProvider::new(
         iter::once(node.peer()).chain(other_elder_peers.clone()),
         Prefix::default(),
+        sk_set0.public_keys(),
     );
 
     let (mut section0, section_key_share) = create_section(&sk_set0, &section_auth0)?;
@@ -1422,6 +1541,8 @@ async fn handle_elders_update() -> Result<()> {
 
     let demoted_peer = other_elder_peers.remove(0);
 
+    let sk_set1 = SecretKeySet::random();
+    let pk1 = sk_set1.secret_key().public_key();
     // Create `HandleAgreement` command for an `OurElders` proposal. This will demote one of the
     // current elders and promote the oldest peer.
     let section_auth1 = SectionAuthorityProvider::new(
@@ -1429,11 +1550,9 @@ async fn handle_elders_update() -> Result<()> {
             .chain(other_elder_peers.clone())
             .chain(iter::once(promoted_peer)),
         Prefix::default(),
+        sk_set1.public_keys(),
     );
-    let elder_names1: BTreeSet<_> = section_auth1.elders.keys().copied().collect();
-
-    let sk_set1 = SecretKeySet::random();
-    let pk1 = sk_set1.secret_key().public_key();
+    let elder_names1: BTreeSet<_> = section_auth1.names();
 
     let proven_section_auth1 = proven(sk_set1.secret_key(), section_auth1)?;
     let proposal = Proposal::OurElders(proven_section_auth1);
@@ -1459,7 +1578,11 @@ async fn handle_elders_update() -> Result<()> {
         let (recipients, message) = match command {
             Command::SendMessage {
                 recipients,
-                message: MessageType::NodeMessage(NodeMessage(msg_bytes)),
+                message:
+                    MessageType::Routing {
+                        msg: RoutingMsg(msg_bytes),
+                        ..
+                    },
                 ..
             } => (recipients, Message::from_bytes(Bytes::from(msg_bytes))?),
             _ => continue,
@@ -1483,10 +1606,10 @@ async fn handle_elders_update() -> Result<()> {
 
     let sync_expected_recipients: HashSet<_> = other_elder_peers
         .into_iter()
-        .map(|peer| *peer.addr())
-        .chain(iter::once(*promoted_peer.addr()))
-        .chain(iter::once(*demoted_peer.addr()))
-        .chain(iter::once(*adult_peer.addr()))
+        .map(|peer| (*peer.name(), *peer.addr()))
+        .chain(iter::once((*promoted_peer.name(), *promoted_peer.addr())))
+        .chain(iter::once((*demoted_peer.name(), *demoted_peer.addr())))
+        .chain(iter::once((*adult_peer.name(), *adult_peer.addr())))
         .collect();
 
     assert_eq!(sync_actual_recipients, sync_expected_recipients);
@@ -1529,6 +1652,7 @@ async fn handle_demote_during_split() -> Result<()> {
     let section_auth_v0 = SectionAuthorityProvider::new(
         iter::once(node.peer()).chain(peers_a.iter().copied()),
         Prefix::default(),
+        sk_set_v0.public_keys(),
     );
 
     let (mut section, section_key_share) = create_section(&sk_set_v0, &section_auth_v0)?;
@@ -1562,14 +1686,18 @@ async fn handle_demote_during_split() -> Result<()> {
     };
 
     // Handle agreement on `OurElders` for prefix-0.
-    let section_auth =
-        SectionAuthorityProvider::new(peers_a.iter().copied().chain(iter::once(peer_c)), prefix0);
+    let section_auth = SectionAuthorityProvider::new(
+        peers_a.iter().copied().chain(iter::once(peer_c)),
+        prefix0,
+        sk_set_v1_p0.public_keys(),
+    );
     let command = create_our_elders_command(sk_set_v1_p0.secret_key(), section_auth)?;
     let commands = dispatcher.handle_command(command).await?;
     assert_matches!(&commands[..], &[]);
 
     // Handle agreement on `OurElders` for prefix-1.
-    let section_auth = SectionAuthorityProvider::new(peers_b.iter().copied(), prefix1);
+    let section_auth =
+        SectionAuthorityProvider::new(peers_b.iter().copied(), prefix1, sk_set_v1_p1.public_keys());
     let command = create_our_elders_command(sk_set_v1_p1.secret_key(), section_auth)?;
     let commands = dispatcher.handle_command(command).await?;
 
@@ -1579,7 +1707,11 @@ async fn handle_demote_during_split() -> Result<()> {
         let (recipients, message) = match command {
             Command::SendMessage {
                 recipients,
-                message: MessageType::NodeMessage(NodeMessage(msg_bytes)),
+                message:
+                    MessageType::Routing {
+                        msg: RoutingMsg(msg_bytes),
+                        ..
+                    },
                 ..
             } => (recipients, Message::from_bytes(Bytes::from(msg_bytes))?),
             _ => continue,
@@ -1593,13 +1725,14 @@ async fn handle_demote_during_split() -> Result<()> {
     let expected_sync_recipients = if prefix0.matches(&node_name) {
         peers_a
             .iter()
-            .map(Peer::addr)
-            .chain(iter::once(peer_c.addr()))
-            .copied()
+            .map(|peer| (*peer.name(), *peer.addr()))
+            .chain(iter::once((*peer_c.name(), *peer_c.addr())))
             .collect()
     } else {
-        // prefix1
-        peers_b.iter().map(Peer::addr).copied().collect()
+        peers_b
+            .iter()
+            .map(|peer| (*peer.name(), *peer.addr()))
+            .collect()
     };
 
     assert_eq!(sync_recipients, expected_sync_recipients);
@@ -1610,7 +1743,7 @@ async fn handle_demote_during_split() -> Result<()> {
 // TODO: add more tests here
 
 #[allow(unused)]
-fn init_log() {
+pub fn init_log() {
     tracing_subscriber::fmt()
         .pretty()
         .with_env_filter(tracing_subscriber::EnvFilter::from_default_env())
